@@ -24,7 +24,8 @@ logger = logging.getLogger(__name__)
 try:
     from PyQt4 import QtGui, QtCore
     from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
-    from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
+    # from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
+    from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar
 except:
     logger.warning('PyQt4 not detected')
     pass
@@ -56,11 +57,31 @@ class sed3:
     # must be same')
 
     def __init__(
-            self, img, voxelsizemm=[1, 1, 1], initslice=0, colorbar=True,
+            self, img, voxelsize=[1, 1, 1], initslice=0, colorbar=True,
             cmap=matplotlib.cm.Greys_r, seeds=None, contour=None, zaxis=0,
             mouse_button_map={1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8},
-            windowW=None, windowC=None, show=False, sed3_on_close=None, figure=None
+            windowW=None, windowC=None, show=False, sed3_on_close=None, figure=None,
+            show_axis=False
     ):
+        """
+
+        :param img:
+        :param voxelsizemm:
+        :param initslice:
+        :param colorbar:
+        :param cmap:
+        :param seeds:
+        :param contour:
+        :param zaxis:
+        :param mouse_button_map:
+        :param windowW:
+        :param windowC:
+        :param show:
+        :param sed3_on_close:
+        :param figure:
+        :param show_axis:
+        :return:
+        """
 
         self.sed3_on_close = sed3_on_close
         self.show_fcn = plt.show
@@ -78,6 +99,8 @@ class sed3:
 
             zaxis = 0
             # pdb.set_trace();
+        self.voxelsize = voxelsize
+        self.actual_voxelsize = copy.copy(voxelsize)
 
         # Rotate data in depndecy on zaxispyplot
         img = self._rotate_start(img, zaxis)
@@ -93,6 +116,7 @@ class sed3:
         self.actual_slice = initslice
         self.colorbar = colorbar
         self.cmap = cmap
+        self.show_axis = show_axis
         if seeds is None:
             self.seeds = np.zeros(self.imgshape, np.int8)
         else:
@@ -112,9 +136,10 @@ class sed3:
                       'btn_view0': "v0", 'btn_view1': "v1", 'btn_view2': "v2"}
 
         # iself.fig.subplots_adjust(left=0.25, bottom=0.25)
-        self.ax = self.fig.add_axes([0.1, 0.20, 0.8, 0.75])
-        # self.ax.set_axis_off()
-        # self.ax.axis('off')
+        if self.show_axis:
+            self.ax = self.fig.add_axes([0.20, 0.25, 0.70, 0.70])
+        else:
+            self.ax = self.fig.add_axes([0.1, 0.20, 0.8, 0.75])
 
         self.ax_colorbar = self.fig.add_axes([0.9, 0.30, 0.02, 0.6])
         self.draw_slice()
@@ -294,9 +319,17 @@ class sed3:
     def _rotate_start(self, data, zaxis):
         if data is not None:
             if zaxis == 0:
-                data = np.transpose(data, (1, 2, 0))
+                tr =(1, 2, 0)
+                data = np.transpose(data, tr)
+                vs = self.actual_voxelsize
+                if self.actual_voxelsize is not None:
+                    self.actual_voxelsize = [vs[tr[0]], vs[tr[1]], vs[tr[2]]]
             elif zaxis == 1:
-                data = np.transpose(data, (2, 0, 1))
+                tr = (2, 0, 1)
+                data = np.transpose(data, tr)
+                vs = self.actual_voxelsize
+                if self.actual_voxelsize is not None:
+                    self.actual_voxelsize = [vs[tr[0]], vs[tr[1]], vs[tr[2]]]
             elif zaxis == 2:
                 # data = np.transpose(data, (0, 1, 2))
                 pass
@@ -310,9 +343,17 @@ class sed3:
         if data is not None:
             if self.rotated_back is False:
                 if zaxis == 0:
-                    data = np.transpose(data, (2, 0, 1))
+                    tr = (2, 0, 1)
+                    data = np.transpose(data, tr)
+                    vs = self.actual_voxelsize
+                    if self.actual_voxelsize is not None:
+                        self.actual_voxelsize = [vs[tr[0]], vs[tr[1]], vs[tr[2]]]
                 elif zaxis == 1:
-                    data = np.transpose(data, (1, 2, 0))
+                    tr = (1, 2, 0)
+                    data = np.transpose(data, tr)
+                    vs = self.actual_voxelsize
+                    if self.actual_voxelsize is not None:
+                        self.actual_voxelsize = [vs[tr[0]], vs[tr[1]], vs[tr[2]]]
                 elif zaxis == 2:
                     pass
                 else:
@@ -356,9 +397,8 @@ class sed3:
                 pass
 
         # self.ax.set_axis_off()
-        self.ax.set_xticklabels([])
-        self.ax.set_yticklabels([])
         # self.ax.set_axis_below(False)
+        self._ticklabels()
 
         # print ctr
         # import pdb; pdb.set_trace()
@@ -369,6 +409,29 @@ class sed3:
 
         # pdb.set_trace();
         # plt.hold(False)
+    def _ticklabels(self):
+        if self.show_axis and self.actual_voxelsize is not None:
+            # pass
+
+            xmax = self.img.shape[0]
+            ymax = self.img.shape[1]
+            xmaxmm = xmax * self.actual_voxelsize[0]
+            ymaxmm = ymax * self.actual_voxelsize[1]
+            xmm = 10.0**np.floor(np.log10(xmaxmm))
+            ymm = 10.0**np.floor(np.log10(ymaxmm))
+            x = xmm * 1.0 / self.actual_voxelsize[0]
+            y = ymm * 1.0 / self.actual_voxelsize[1]
+
+            self.ax.set_xticks([0, x])
+            self.ax.set_xticklabels([0, xmm])
+            self.ax.set_yticks([0, y])
+            self.ax.set_yticklabels([0, ymm])
+            # self.ax.set_yticklabels([13,12])
+
+        else:
+            self.ax.set_xticklabels([])
+            self.ax.set_yticklabels([])
+
 
     def next_slice(self):
         self.actual_slice = self.actual_slice + 1
@@ -512,7 +575,7 @@ class sed3:
 
 
 def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
-                shape=None, show=True):
+                shape=None, show=True, flipH=False, flipV=False):
     """
     Show slices as tiled image
 
@@ -568,7 +631,7 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
 
         slim = __put_slice_in_slim(slim, im2d, sh, i)
     #         show_slice(im2d, cont, seeds2d)
-    show_slice(slim, slco, slse)
+    show_slice(slim, slco, slse, flipH=flipH, flipV=flipV)
     if show:
         plt.show()
 
@@ -618,9 +681,35 @@ def __put_slice_in_slim(slim, dataim, sh, i):
 #     plt.close()
 
 
-def show_slice(data2d, contour2d=None, seeds2d=None):
+def show_slice(data2d, contour2d=None, seeds2d=None, flipV=False, flipH=False):
+    """
+
+    :param data2d:
+    :param contour2d:
+    :param seeds2d:
+    :param flipV: vertical flip
+    :param flipH: horizontal flip
+    :return:
+    """
     import copy as cp
     # Show results
+    if flipV:
+
+        if data2d is not None:
+            data2d = data2d[-1:0:-1,:]
+        if seeds2d is not None:
+            seeds2d = seeds2d[-1:0:-1,:]
+        if contour2d is not None:
+            contour2d= contour2d[-1:0:-1,:]
+    if flipH:
+        if data2d is not None:
+            data2d = data2d[:, -1:0:-1]
+        if seeds2d is not None:
+            seeds2d = seeds2d[:, -1:0:-1]
+        if contour2d is not None:
+            contour2d= contour2d[:, -1:0:-1]
+        # data2d = data2d[:, ::-1]
+
     colormap = cp.copy(plt.cm.get_cmap('brg'))
     colormap._init()
     colormap._lut[:1:, 3] = 0
@@ -638,6 +727,10 @@ def show_slice(data2d, contour2d=None, seeds2d=None):
 
 
 def __select_slices(data, axis, slice_step):
+    if data is None:
+        return None
+
+
     if axis == 0:
         data = data[::slice_step, :, :]
     if axis == 1:
@@ -830,6 +923,8 @@ if __name__ == "__main__":
         default='output.mat', help='output file name')
     args = parser.parse_args()
 
+    voxelsize = None
+
     if args.debug:
         logger.setLevel(logging.DEBUG)
 
@@ -839,7 +934,8 @@ if __name__ == "__main__":
         unittest.main()
 
     if args.example3d:
-        data = generate_data()
+        data = generate_data([16, 20, 24])
+        voxelsize = [0.1, 1.2, 2.5]
     elif args.filename == 'lena':
         from scipy import misc
 
@@ -864,7 +960,7 @@ if __name__ == "__main__":
         # klávesnice
         # tě ebo jinak
 
-    pyed = sed3(data)
+    pyed = sed3(data, voxelsize=voxelsize)
     output = pyed.show()
 
     scipy.io.savemat(args.outputfile, {'data': output})
